'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = insert;

var _getOutputImage = require('../internal/getOutputImage');

var _getImageParameters = require('../internal/getImageParameters');

var _getImageParameters2 = _interopRequireDefault(_getImageParameters);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Inserts an image within another image.
 * @memberof Image
 * @instance
 * @param {Image} toInsert The image to insert. Out of boundary pixel will be ignored.
 * @param {object} [options]
 * @param {number} [options.x=0] x offset
 * @param {number} [options.y=0] y offset
 * @param {boolean} [options.inPlace=false] - If true modifies the image. If false the insertion is performed on a copy of the image.
 * @return {Image} The modified image or the new image.
 */
function insert(toInsert, options = {}) {
  var parameters = (0, _getImageParameters2.default)(toInsert);
  this.checkProcessable('insert', parameters);
  var _options$x = options.x,
      x = _options$x === undefined ? 0 : _options$x,
      _options$y = options.y,
      y = _options$y === undefined ? 0 : _options$y;


  var out = (0, _getOutputImage.getOutputImageOrInPlace)(this, options, { copy: true });
  var maxY = Math.min(out.height, y + toInsert.height);
  var maxX = Math.min(out.width, x + toInsert.width);
  if (out.bitDepth === 1) {
    for (var j = y; j < maxY; j++) {
      for (var i = x; i < maxX; i++) {
        var val = toInsert.getBitXY(i - x, j - y);
        if (val) out.setBitXY(i, j);else out.clearBitXY(i, j);
      }
    }
  } else {
    for (var _j = y; _j < maxY; _j++) {
      for (var _i = x; _i < maxX; _i++) {
        out.setPixelXY(_i, _j, toInsert.getPixelXY(_i - x, _j - y));
      }
    }
  }

  return out;
}